﻿using System;
using System.Collections.Generic;
using System.Text;

namespace PerfectCS_2012
{
    class MusicAlbum
    {
        // -------- ①コンストラクタ
        public MusicAlbum(string title, string artist, Music[] musics)
        {
            m_title = title;
            m_artist = artist;
            m_musics = musics;
        }
        // -------- ②フィールド
        private string m_title;     // -- タイトル
        private string m_artist;    // -- アーチスト
        private Music[] m_musics;   // -- ミュージックリスト
        // -------- ③プロパティ
        public string Title{    get { return m_title; }}
        public string Artist{   get { return m_artist; }}
        public Music[] Musics
        {
            get { return m_musics; }
            set { m_musics = value; }
        }
        public int PlayingSeconds
        {
            get
            {
                int playingSeconds = 0;
                foreach (Music music in this.Musics){   playingSeconds += music.PlayingSeconds;}
                return playingSeconds;
            }
        }
        public string PlayingTime
        {
            get
            {
                int playingSeconds = this.PlayingSeconds;
                string minitsParts = (playingSeconds / c_minuteSeconds).ToString() + c_minutesModification;
                string secondParts = (playingSeconds % c_minuteSeconds).ToString() + c_secondsModification;
                return minitsParts + secondParts;
            }
        }
        // -------- ④定数
        private const int c_minuteSeconds = 60;
        private const string c_minutesModification = "分";
        private const string c_secondsModification = "秒";
        // -------- ⑤メソッド
        public bool IsMusicContained(string musicTitle)
        {
            foreach (Music music in this.Musics)
            {
                if(music.Title == musicTitle){  return true;}
            }
            return false;
        }
        public bool IsMusicContained(string musicTitle, string musicArtist)
        {
            foreach (Music music in this.Musics) 
            {
                if (music.Title == musicTitle && music.Artist == musicArtist) { return true; } 
            }
            return false;
        }
        public void AddMusic(Music music)
        {
            int lastMusicCount = m_musics.Length;
            Array.Resize(ref m_musics, lastMusicCount + 1);
            m_musics[lastMusicCount] = music;
        }
        // -------- ⑥インデクサ
        public Music this[int index]
        {
            get { return m_musics[index]; }
            set { m_musics[index] = value; }
        }
        // -------- デリゲート
        public delegate bool CheckMusicDelegate(int musicIndex, Music music);
        // --------
        public void CheckAlbum(CheckMusicDelegate checkMusicDelegate)
        {
            int musicIndex = 0;
            while (musicIndex < this.Musics.Length)
            {
                Music music = this[musicIndex];
                if (checkMusicDelegate != null)
                {
                    bool isBreak = checkMusicDelegate(musicIndex, music);
                    if (isBreak){   break;}
                }
                musicIndex++;
            }
        }
    }
}
