﻿//-#define THREAD
//-#define BACKGROUNDWORKER
//-#define TASK
//-#define TASK_ContinueWith
//-#define TASK_WaitAny
//-#define PARALLEL
#define ASYNC

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
// --------
using System.Threading;
using System.ComponentModel;
// --------
namespace PerfectCS_2012
{
    // --------
    class Program
    {
        static void Main(string[] args)
        {
#if PARALLEL
            // ---- ■Parallel(Invoke)
            Parallel.Invoke(
                () => { new Counter("▼Task", 2).Play(); },
                () => { new Counter("▼▼Task", 3).Play(); },
                () => { new Counter("▼▼▼Task", 4).Play(); });

            new Counter("▽Main", 3).Play();
            // ----
            Console.ReadLine();
#endif

#if TASK_WaitAny
            // ---- ■Task(WaitAny, WaitAll)
            Task task1 = Task.Factory.StartNew(() => { new Counter("▼Task", 2).Play(); });
            Task task2 = Task.Factory.StartNew(() => { new Counter("▼▼Task", 3).Play(); });
            Task task3 = Task.Factory.StartNew(() => { new Counter("▼▼▼Task", 4).Play(); });

            Task.WaitAny(task1, task2, task3);
            new Counter("▽Main", 3).Play();

            Task.WaitAll(task1, task2, task3);
            new Counter("▽▽Main", 3).Play();
            // ----
            Console.ReadLine();
#endif


#if ASYNC
            // ---- ■async - await
            Task task = Program.PlayAsync();
            new Counter("▽Main", 3).Play();
            task.Wait();
            // ----
            new Counter("▽▽Main", 3).Play();
            // ----
            Console.ReadLine();
#endif

#if TASK || TASK_ContinueWith
            // ---- ■Task
            Task task = Task.Factory.StartNew(() => { new Counter("▼Task", 5).Play(); });
            //-Task task = new TaskFactory().StartNew(() => { new Counter("▼Task", 5).Play(); });

            new Counter("▽Main", 3).Play();
#if TASK_ContinueWith
            // ----
            Task continueTask = task.ContinueWith((Task thisTask) => { new Counter("▼▼Task", 3).Play(); });
            continueTask.Wait();
#else
            task.Wait();
#endif
            new Counter("▽▽Main", 3).Play();
            // ----
            Console.ReadLine();
#endif
#if BACKGROUNDWORKER
            // ---- ■BackgroundWorker
            BackgroundWorker backgroundWorker = new BackgroundWorker();
            backgroundWorker.DoWork += new DoWorkEventHandler(new Counter("▼Thread", 5).BackgroundWorker_DoWork);
            backgroundWorker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(new Counter("▽▽Main", 3).BackgroundWorker_RunWorkerCompleted);
            // ----
            backgroundWorker.RunWorkerAsync();
            // ----
            new Counter("▽Main", 3).Play();

            // ----
            Console.ReadLine();
#endif

#if THREAD
            // ---- ■Thread
            object wait = new object();
            // ----
            new Counter("▼Thread", 5).ThreadPlay(wait);
            new Counter("▽Main", 3).Play();
            lock (wait) { new Counter("▽▽Main", 3).Play(); }
            // ----
            Console.ReadLine();
#endif
        }


#if ASYNC
        // --------
        static async Task PlayAsync()
        {
            await Task.Run(() => { new Counter("▼Task", 5).Play(); });
            // ----
            new Counter("▼▼Task", 3).Play();
            // ----
        }
#endif
    }


    // --------
    class Counter
    {
        // -------- ●
        public Counter(string caption, int count)
        {
            this.Caption = caption;
            this.Count = count;
        }
        public string Caption { get; set; }
        public int Count { get; set; }
        // --------
        public void Play()
        {
            int count = 0;
            while (count < this.Count)
            {
                count++;
                Console.WriteLine("{0}({1}/{2})", this.Caption, count.ToString(), this.Count.ToString());
                Thread.Sleep(100);
            }
        }
#if THREAD
        // --------
        public void ThreadPlay(object wait)
        {
            ParameterizedThreadStart threadStart = new ParameterizedThreadStart(this.WaitPlay);
            Thread thread = new Thread(threadStart);
            thread.Start(wait);
        }
        // --------
        public void WaitPlay(object wait)
        {
            if (wait != null) { lock (wait) { this.Play(); } }
            else { this.Play(); }
        }
#endif
#if BACKGROUNDWORKER
        // --------
        public void BackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            this.Play();
        }
        // --------
        public void BackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.Play();
        }
#endif
    }
}
