﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;

namespace WpfApplication
{
    /// <summary>
    /// CallModelessWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class CallModelessWindow : Window
    {
        public CallModelessWindow()
        {
            InitializeComponent();

            // タイマーを作成
            _dt = new DispatcherTimer(DispatcherPriority.Background);
            _dt.Tick += new EventHandler(_dt_Tick);
            _dt.Interval = TimeSpan.FromMilliseconds(1000);
            _dt.Start();
        }

        // カウンタ値
        private int _counter = 0;

        // タイマオブジェクトプロパティ
        DispatcherTimer _dt;

        // モードレスウィンドウからの通知イベントハンドラ
        void timeSpanWindow_TimeSpanChanged(object sender, EventArgs e)
        {
            // senderはイベント発生元であるTimeSpanWindowオブジェクト
            TimeSpanWindow timeSpanWindow = sender as TimeSpanWindow;
            // getアクセサで値を取得しタイマに反映
            _dt.Interval = TimeSpan.FromMilliseconds(timeSpanWindow.TimeSpan);
        }

        // タイマイベント
        void _dt_Tick(object sender, EventArgs e)
        {
            // カウントアップ
            tbCounter.Text = (++_counter).ToString();
        }

        // モードレス表示ボタンクリックイベントハンドラ
        private void btnShowModeless_Click(object sender, RoutedEventArgs e)
        {
            // すでにTimeSpanWindowが存在したらフォーカスを移すだけ
            foreach (Window window in this.OwnedWindows)
            {
                if (window is TimeSpanWindow)
                {
                    window.Focus();
                    return;
                }
            }

            // モードレスウィンドウ作成
            TimeSpanWindow timeSpanWindow = new TimeSpanWindow();

            // オーナーウィンドウ作成
            timeSpanWindow.Owner = this;

            // イベントハンドラ登録
            timeSpanWindow.TimeSpanChanged += new TimeSpanChangedEventHandler(timeSpanWindow_TimeSpanChanged);

            // モードレス表示
            timeSpanWindow.Show();
        }
    }
}