﻿using System;
using System.Windows;
using Microsoft.Win32;

namespace WpfApplication
{
    /// <summary>
    /// FileOpenDialogSample.xaml の相互作用ロジック
    /// </summary>
    public partial class FileOpenDialogSample : Window
    {
        public FileOpenDialogSample()
        {
            InitializeComponent();
        }

        private void btnOpenFile_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog fileDlg = new OpenFileDialog();
            // ファイルダイアログの初期ディレクトリを設定
            fileDlg.InitialDirectory =
            System.Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            // 選択したファイルの存在チェックを有効にする
            fileDlg.CheckFileExists = true;
            // 読み取り専用チェックボックスの表示
            fileDlg.ShowReadOnly = true;
            // ファイルの種類フィルタの設定
            fileDlg.Filter = "Wordドキュメント|*.docx|Excelワークシート|*.xlsx|Officeファイル|*.docx;*.xlsx;*.pptx;*.accdb|すべてのファイル|*.*";
            // 開くボタンクリック時のイベントハンドラ設定
            fileDlg.FileOk += new System.ComponentModel.CancelEventHandler(fileDlg_FileOk);
            bool? result = fileDlg.ShowDialog();
            if (true==result)
            {
                // 開くボタンによってダイアログが閉じられた
                MessageBox.Show("フルパス:" + fileDlg.FileName +
                " ファイル名:" + fileDlg.SafeFileName);
            }
        }

        // ファイルオープンダイアログの開くボタンクリックイベントハンドラ
        void fileDlg_FileOk(object sender, System.ComponentModel.CancelEventArgs e)
        {
            // イベント発生元のOpenFileDialogオブジェクトを取得
            OpenFileDialog dlg = sender as OpenFileDialog;
            if ("InvalidFileName.docx" == dlg.SafeFileName)
                e.Cancel = true; // Cancelをtrueに設定するとファイルダイアログは開いたままになる
        }
    }
}
