﻿using System;
using System.Windows.Forms;

using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;

namespace Fundamental
{
    public partial class SampleForm : Form
    {
        public SampleForm()
        {
            InitializeComponent();
        }

        private void btnConnDisconn_Click(object sender, EventArgs e)
        {
            // 接続オブジェクトの作成
            SqlConnection conn = new SqlConnection(
                "Data Source=localhost;Initial Catalog=AdventureWorks;" +
                "Integrated Security=True");
            // 接続をオープン
            conn.Open();

            lstOutput.Items.Insert(0, "接続を開きました");

            //...データベースアクセス処理

            // 接続を閉じる
            conn.Close();

            lstOutput.Items.Insert(0, "接続を閉じました");

#if false
            SqlConnection conn = null;
            try
            {
                conn = new SqlConnection(
                    "Data Source=localhost;Initial Catalog=AdventureWorks;" +
                    "Integrated Security=True");
                // ...データベースアクセス処理
            }
            finally {
                if( conn != null )
                conn.Close();
            }
#endif
#if false
            using( SqlConnection conn = new SqlConnection( "Data Source=localhost;Initial Catalog=AdventureWorks;Integrated Security=True") ) 
            {
                // ...データベースアクセス処理
            }
#endif
        }

        private void btnSqlDataReader_Click(object sender, EventArgs e)
        {
            // 接続オブジェクトの作成
            using (var conn = new SqlConnection(
                "Data Source=localhost;Initial Catalog=AdventureWorks;" +
                "Integrated Security=True"))
            {
                // コマンドオブジェクトの作成
                SqlCommand cmd = new SqlCommand(
                "SELECT BusinessEntityID, Name, ModifiedDate FROM Sales.Store", conn);
                // データベースとの接続
                conn.Open();
                // コマンド実行
                using (SqlDataReader reader = cmd.ExecuteReader())
                {
                    // データの読み出し
                    while (reader.Read())
                    {
                        // 抽出データの取得
                        int businessEntityID = (int)reader["BusinessEntityID"];
                        string name = (string)reader["Name"];

                        Debug.WriteLine(businessEntityID.ToString() + " " + name);
                        this.lstOutput.Items.Insert(0, businessEntityID.ToString() + " " + name);
                    }
                }
            }
        }

        private void btnDataSet_Click(object sender, EventArgs e)
        {
            DataSet ds = null;
            // 接続オブジェクトの作成
            using (var conn = new SqlConnection(
                "Data Source=localhost;Initial Catalog=AdventureWorks;" +
                "Integrated Security=True"))
            {
                // データアダプタオブジェクトの作成
                SqlDataAdapter adapter = new SqlDataAdapter("SELECT * FROM Sales.Store", conn);

                // DataSetオブジェクトの作成
                ds = new DataSet();

                // データベースとの接続
                conn.Open();

                // データ抽出処理
                adapter.Fill(ds);
            }
            // 取得したデータを参照
            for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
            {
                int businessEntityID = (int)ds.Tables[0].Rows[i]["BusinessEntityID"];
                string name = (string)ds.Tables[0].Rows[i]["Name"];

                Debug.WriteLine(businessEntityID.ToString() + " " + name);
                this.lstOutput.Items.Insert(0, businessEntityID.ToString() + " " + name);
            }
        }

        private void btnGetScaler_Click(object sender, EventArgs e)
        {
            using (var conn = new SqlConnection(
                "Data Source=localhost;Initial Catalog=AdventureWorks;" +
                "Integrated Security=True"))
            {
                SqlCommand cmd = new SqlCommand(
                "SELECT count(*) FROM Sales.Store", conn);

                conn.Open();

                int recCount = (int)cmd.ExecuteScalar();

                this.lstOutput.Items.Insert(0, recCount.ToString());
            }
        }

        private void btnUseParameter_Click(object sender, EventArgs e)
        {
            using (var conn = new SqlConnection(
                "Data Source=localhost;Initial Catalog=AdventureWorks;" +
                "Integrated Security=True"))
            {
                SqlCommand cmd = new SqlCommand(
                    "SELECT * FROM Sales.Store WHERE businessEntityID = @BusinessEntityID", conn);
                cmd.Parameters.Add(new SqlParameter("@BusinessEntityID", 2051));
                conn.Open();
                using (SqlDataReader reader = cmd.ExecuteReader())
                {
                    if (reader.Read())
                    {
                        int businessEntityID = (int)reader["BusinessEntityID"];
                        string name = (string)reader["Name"];

                        Debug.WriteLine(businessEntityID.ToString() + " " + name);
                        this.lstOutput.Items.Insert(0, businessEntityID.ToString() + " " + name);
                    }
                }
            }
        }

        private void btnCommandInsert_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this, "INSERT処理を行ってよろしいですか？", "", MessageBoxButtons.YesNo) == System.Windows.Forms.DialogResult.Yes)
            {
                InsertCulture("99999", "test", DateTime.Now);
            }
        }

        public int InsertCulture(string cultureID, string name, DateTime modifiedDate)
        {
            int affected = 0;

            using (var conn = new SqlConnection(
                "Data Source=localhost;Initial Catalog=AdventureWorks;" +
                "Integrated Security=True"))
            {
                SqlCommand cmd = new SqlCommand(
                "INSERT INTO Production.Culture (CultureID, Name, ModifiedDate) " +
                "VALUES (@CultureID, @Name, GETDATE())", conn);
                cmd.Parameters.Add("@CultureID", SqlDbType.NChar);
                cmd.Parameters.Add("@Name", SqlDbType.NVarChar);
                cmd.Parameters["@CultureID"].Value = cultureID;
                cmd.Parameters["@Name"].Value = name;
                conn.Open();
                affected = cmd.ExecuteNonQuery();
            }
            return affected;
        }

        private void btnUpdateDataSet_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this, "INSERT/UPDATE/DELETE処理を行ってよろしいですか？", "", MessageBoxButtons.YesNo) == System.Windows.Forms.DialogResult.No)
                return;
            
            using (var conn = new SqlConnection("Data Source=localhost;Initial Catalog=AdventureWorks;Integrated Security=True"))
            {
                SqlDataAdapter adapter =
                new SqlDataAdapter(
                "SELECT * FROM Production.Culture",
                conn);
                // 挿入コマンドを設定
                adapter.InsertCommand =
                new SqlCommand(
                "INSERT INTO Production.Culture (CultureID, Name, ModifiedDate) " +
                "VALUES (@CultureID, @Name, GETDATE())",
                conn);
                SqlParameter cultureParam = new SqlParameter("@CultureID", SqlDbType.NChar);
                cultureParam.SourceColumn = "CultureID";
                adapter.InsertCommand.Parameters.Add(cultureParam);
                SqlParameter nameParam =
                new SqlParameter("@Name", SqlDbType.NVarChar);
                nameParam.SourceColumn = "Name";
                adapter.InsertCommand.Parameters.Add(nameParam);
                // 更新コマンドを設定
                adapter.UpdateCommand =
                new SqlCommand(
                "UPDATE Production.Culture SET Name = @Name, " +
                "ModifiedDate = GETDATE() WHERE CultureID = @CultureID",
                conn);
                cultureParam = new SqlParameter("@CultureID", SqlDbType.NChar);
                cultureParam.SourceColumn = "CultureID";
                adapter.UpdateCommand.Parameters.Add(cultureParam);
                nameParam = new SqlParameter("@Name", SqlDbType.NVarChar);
                nameParam.SourceColumn = "Name";
                adapter.UpdateCommand.Parameters.Add(nameParam);
                // 削除コマンドを設定
                adapter.DeleteCommand =
                new SqlCommand(
                "DELETE FROM Production.Culture " +
                "WHERE CultureID = @CultureID",
                conn);
                cultureParam = new SqlParameter("@CultureID", SqlDbType.NChar);
                cultureParam.SourceColumn = "CultureID";
                adapter.DeleteCommand.Parameters.Add(cultureParam);

                // データベース→DataSet（データを取得）
                // （第2引数によりDataSet内のDataTableオブジェクトのテーブル名を設定）
                DataSet ds = new DataSet();
                adapter.Fill(ds, "Culture");
                // 新規行を追加
                DataRow newRow = ds.Tables["Culture"].NewRow();
                newRow["CultureID"] = "ja";
                newRow["Name"] = "japanese";
                newRow["ModifiedDate"] = DateTime.Today;
                ds.Tables["Culture"].Rows.Add(newRow);
                // 既存行を更新
                ds.Tables["Culture"].Rows[3]["Name"] = "modify";
                // 既存行を削除
                ds.Tables["Culture"].Rows[0].Delete();
                // DataSet → データベース
                adapter.Update(ds, "Culture");
            }
        }
    }
}